use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Anirvan
Chatterjee
DROLSKY
DROLSKY's
Data
Dave
David
Domain
Felipe
Gasper
Gregory
Neely
Neil
Oschwald
PayPal
Rolsky
Rolsky's
Sonnen
Steinbrunner
TLD
TLDs
Validate
anirvan
autarch
cx
drolsky
dsteinbrunner
felipe
gTLDs
goschwald
lib
neely
neil
uk
www
